;-----------------------------------------------------------------------
; AutoHotkey script to create an animated GIF from photobooth photos.
;
; Usage: Double click on this script to install it (this updates the PhotoboothStatusCmdXML
; setting in the Windows registry). Double click on the script again if you wish to uninstall it.
; Once installed run the photobooth software as normal and the script will be run automtically
; and will save the animated GIFs in a subfolder named GIF.
; Edit the resizeWidth, titleDelay and imageDelay settings below to configure the way the animated
; GIF is created.
;
; A title screen can be displayed (after the photos) by saving a file named GIF_title.jpg in the
; photobooth images folder. Please note the GIF_title.jpg image must have the same aspect ratio as
; the photos otherwise you may see parts of the last photo when the title screen is displayed.
;
; An optional overlay can be added to each photo by saving a file named GIF_overlay_1.png,
; GIF_overlay_2.png, GIF_overlay_3.png etc. in the photobooth images folder. If the numbered file
; isn't found the script will look for GIF_overlay.png instead.
; To animate the overlay at a faster frame rate than the photos set overlaysPerPhoto to the number of
; overlay frames that should be displayed per photo.
;
; An optional background can be added to each photo by saving a file named GIF_background_1.jpg,
; GIF_background_2.jpg, GIF_background_3.jpg etc. in the photobooth images folder. If the numbered file
; isn't found the script will look for GIF_background.jpg instead.
; To animate the background at a faster frame rate than the photos set overlaysPerPhoto to the number of
; background frames that should be displayed per photo.
; To add backgrounds to greenscreened images the photobooth software should be setup to save greenscreened
; copies of the photos as transparent PNG images.
;
; MP4 movie files: This script can also create an MP4 movie file. To enable this first download
; FFmpeg from https://www.ffmpeg.org. Then set createMP4copy to 1 and set ffmpegDir to the folder
; where FFmpeg is installed. When the script runs it will create the animated GIF file and an MP4
; movie file.
;
; Reverse display: set reverseDisplay to 1 to display the middle photos in reverse order at the end of
; the loop e.g. 4 photos displayed as photo 1, photo 2, photo 3, photo 4, photo 3, photo 2
;
; Green screen support: the script looks for a JPG or PNG copy of the photo in the greenscreen subfolder
; and uses that, if present, otherwise it uses the unmodified photo taken by the camera.
;
; Crop to square: set cropToSquare to 1 to crop photos to square format. Each photo will be cropped
; to resizeWidth x resizeWidth pixels.
;
; AutoHotKey is free and can be downloaded from:
; http://www.autohotkey.com
;
; This script comes with no warranty or support whatsoever and may
; be freely copied or modified as required.
;
; Written by Chris Breeze, www.breezesys.com
; 20141019: Updated to handle pathnames containing spaces
; 20150318: Added cropToSquare and createMP4copy options
; 20150427: Added reverseDisplay option to display photos as 1, 2, 3, 4, 3, 2
; 20151118: Added optional backgrounds and processing of transparent PNG green screen images
; 20151221: videoStillsAuto - Read playback rate and "ping-pong" playback settings from XML file
; 20160426: updated videoStillsAuto to read image_delay from XML file (DSLR Remote Pro v3.5 onwards)
; 20160507: Added rotate option
; 20160524: MP4 file now created from intermediate PNG images for better color. Added minMp4duration
;           and mp4OutputDir
;-----------------------------------------------------------------------
#SingleInstance,force
#NoEnv
SetWorkingDir %A_ScriptDir%

; resize photos to this width (in pixels)
resizeWidth := 400

; auto mode for DSLR Remote Pro stills capture in video booth mode
; reads playback rate and "ping-pong" playback settings from XML summary file
videoStillsAuto := 1

; crop image to resizeWidth x resizeWidth square if non-zero
cropToSquare := 1

; rotate images CW (in degrees) e.g. for portrait set this to 90 or 270
rotate := 90

; set reverseDisplay to 1 to add middle photos to the end of the loop in reverse order e.g. 1, 2, 3, 4, 3, 2
reverseDisplay := 0

; how long to display title screen (in 1/100 sec)
titleDelay := 200

; how long to display each photo (in 1/100 sec)
imageDelay := 60

; Subfolder name where GIF is saved
outputDir := "GIF"

; Subfolder name where MP4 is saved
mp4OutputDir := "MP4"

; The number of overlay animations to display per photo (set this to 1 if there are no overlays)
overlaysPerPhoto := 1

; create MP4 copy of animated GIF
createMP4copy := 1
ffmpegDir = D:\ffmpeg\ffmpeg-20160404-git-54c9146-win64-static

; set this to the minimum duration (in 1/100 sec) for the MP4
; e.g. set it to 300 to create a 3 sec video suitable for posting on Instagram
minMP4duration := 300

if createMP4copy
{
	ifnotexist,%ffmpegDir%\bin\ffmpeg.exe
	{
		MsgBox, ffmpeg not found, please check ffmpeg is installed and ffmpegDir is defined in the script.`nffmpegDir=%ffmpegDir%
		exitapp
	}
	FileDelete,temp_png_*.png
}

; set this to 1 to enable debug messages or 0 to disable them
showDebugMessages := 0

if showDebugMessages
{
	Gui +LastFound +AlwaysOnTop +ToolWindow  ; +ToolWindow avoids a taskbar button and an alt-tab menu item.
	Gui, Color, cBlack
	Gui, Font, s12 
	Gui, Add, Text, vStatusText cLime, AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	Gui, Add, Text, vStatusText2 cLime, AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	Gui, Show,x100 y0 NoActivate,%A_ScriptName%  ; NoActivate avoids deactivating the currently active window.
	GuiControl,,StatusText,Please wait...
	GuiControl,,StatusText2,
}

if cropToSquare
{
	resizeCmd = -resize "%resizeWidth%x%resizeWidth%^" -crop %resizeWidth%x%resizeWidth%+0+0 +repage
}
else
{
	resizeCmd = -resize %resizeWidth%
}

if rotate
{
	resizeCmd = -rotate %rotate% %resizeCmd%
}

IfNotExist, %1%
{
	; no file passed via command line: install or uninstall the script
	Install()
	exitapp
}

; Extract filename and directory from XML pathname
dir = %1%
DebugMessage(1, "Reading: " . dir)
RegExMatch(dir, "([^\\]+)$", filename) 
RegExMatch(dir, "(.*)\\", dir) 

SetWorkingDir %dir%
FileCreateDir %outputDir%

; read info from XML file
FileRead, Contents, %1%
RegExMatch(Contents, "<date>(.*)</date>", date_taken) 
RegExMatch(Contents, "<time>(.*)</time>", time_taken) 
RegExMatch(Contents, "<prints>(.*)</prints>", prints) 
RegExMatch(Contents, "<user_data>(.*)</user_data>", email) 
RegExMatch(Contents, "<user_data2>(.*)</user_data2>", name) 
RegExMatch(Contents, "<bw_mode>(.*)</bw_mode>", bw) 
RegExMatch(Contents, "<photobooth_images_folder>(.*)</photobooth_images_folder>", photoboothFolder) 

if videoStillsAuto
{
	; read video mode stills capture parameters from XML summary file
	IfInString,Contents,<video_information>
	{
		RegExMatch(Contents, "<evf_interval_ms>(.*)</evf_interval_ms>", evfIntervalMS) 
		RegExMatch(Contents, "<playback_freq>(.*)</playback_freq>", playbackFreq) 
		RegExMatch(Contents, "<image_delay>(.*)</image_delay>", playbackImageDelay) 
		RegExMatch(Contents, "<ping_pong_playback>(.*)</ping_pong_playback>", pingPongPlayback) 
		if playbackImageDelay1 > 0
		{
			imageDelay := playbackImageDelay1
			reverseDisplay := pingPongPlayback1
		}
		else if evfIntervalMS1 > 10
		{
			if playbackFreq1 > 0
			{
				imageDelay := Round((evfIntervalMS1 * playbackFreq1) / 10)
				reverseDisplay := pingPongPlayback1
			}
		}
	}
}

; Look for title page named GIF_title.jpg or title.jpg
title = %photoboothFolder1%\GIF_title.jpg
IfNotExist,%title%
{
	title = %photoboothFolder1%\title.jpg
}

; Add photos to 'files' array
files = 0
while 1
{
	pattern = <photo image="%A_Index%">(.*)\.(.*)</photo>
	if RegExMatch(Contents, pattern, photo) 
	{
		; check whether we should use the greenscreened copy of the photo
		ifExist,greenscreen\%photo1%.jpg
		{
			photo1 = greenscreen\%photo1%.jpg
		}
		else ifExist,greenscreen\%photo1%.png
		{
			photo1 = greenscreen\%photo1%.png
		}
		else
		{
			photo1 = %photo1%.%photo2%
		}
		files%A_Index% := photo1
		files++
	}
	else
	{
		break
	}
}

; optionally add files in reverse order e.g. 1, 2, 3, 4, 3, 2
if reverseDisplay
{
	filesToAdd := files - 1
	while filesToAdd > 1
	{
		files++
		files%files% := files%filesToAdd%
		filesToAdd--
	}
}

; add each photo in 'files' array to command line
counter := 0
loop %files%
{
	photoNumber := A_Index
	photo := files%A_Index%
	DebugMessage(2, photoNumber . ": " . photo)
	logo_overlay = %photoboothFolder1%\GIF_overlay_logo_%photoNumber%.png
	IfNotExist,%logo_overlay%
	{
		logo_overlay = %photoboothFolder1%\GIF_overlay_logo.png
	}

	loop, %overlaysPerPhoto%
	{
		frameImages :=
		breakOutOfLoop := 1
		index := overlaysPerPhoto * (photoNumber - 1) + A_Index

		; add optional background
		background = %photoboothFolder1%\GIF_background_%index%.jpg
		IfExist,%background%
		{
			; don't break out of loop if we find a numbered background image
			breakOutOfLoop := 0
		}
		else
		{
			; use GIF_background.png if GIF_background_n.png not found
			background = %photoboothFolder1%\GIF_background.jpg
		}
		IfExist,%background%
		{
			frameImages = %frameImages% "%background%"
		}

		; add the photo
		frameImages = %frameImages% "%photo%"

		; add optional overlay
		overlay = %photoboothFolder1%\GIF_overlay_%index%.png
		IfExist,%overlay%
		{
			; don't break out of loop if we find a numbered overlay image
			breakOutOfLoop := 0
		}
		else
		{
			; use GIF_overlay.png if GIF_overlay_n.png not found
			overlay = %photoboothFolder1%\GIF_overlay.png
		}
		IfExist,%overlay%
		{
			frameImages = %frameImages% "%overlay%"
		}

		; add optional logo_overlay
		IfExist,%logo_overlay%
		{
			frameImages = %frameImages% "%logo_overlay%"
		}

		if createMP4copy
		{
			; create PNG copies of each frame so that MP4 has full color (not the GIF's 256 colors)
			counter := counter + 1
			cmd = "%A_ScriptDir%\convert.exe" -gravity center
			if (bw1 = 1)
			{
				cmd = %cmd% -grayscale Rec601Luma
			}
			cmd = %cmd% ( %frameImages% %resizeCmd% -flatten ) temp_png_%counter%.png
			runwait %cmd%,, Hide
		}

		images = %images% ( %frameImages% %resizeCmd% -flatten )
		images := images . " "

		if breakOutOfLoop
		{
			; frames don't have animated backgrounds or overlays
			break
		}
	}
}

; construct the ImageMagick convert command
cmd = "%A_ScriptDir%\convert.exe" -gravity center

; loop forever and add the photos
delay := imageDelay / overlaysPerPhoto
cmd = %cmd% -loop 0 -delay %delay% %images%

; convert to B&W if bw_mode=1
if (bw1 = 1)
{
	cmd = %cmd% -grayscale Rec601Luma
}

; add title screen if defined
ifExist, %title%
{
	cmd = %cmd% -delay %titleDelay% "%title%"
}

; resize images and save as a GIF in a temporary file
output := SubStr(filename,1,-4)
cmd = %cmd% -resize %resizeWidth% temp.gif

; run the command and then move temporary GIF file to %gifPathName%
runwait %cmd%,, Hide
FileMove,temp.gif,%outputDir%\%output%.gif

if createMP4copy
{
	FileCreateDir %mp4OutputDir%
	duration := 0
	inputFiles :=
	loop
	{
		; add title screen if defined
		ifExist, %title%
		{
			if inputFiles
			{
				inputFiles = %inputFiles%|%title%
			}
			else
			{
				inputFiles = %title%
			}
			duration := duration + delay
		}
		loop,temp_png_*.png
		{
			if inputFiles
			{
				inputFiles = %inputFiles%|%A_LoopFilename%
			}
			else
			{
				inputFiles = %A_LoopFilename%
			}
			duration := duration + delay
		}

		; is it long enough yet?
		if (duration >= minMP4duration)
		{
			break
		}
	}

	FileDelete,%mp4OutputDir%\%output%.mp4
	cmd = %ffmpegDir%\bin\ffmpeg.exe -framerate 100/%delay% -i concat:%inputFiles% -vf "scale=trunc(iw/2)*2:trunc(ih/2)*2" -r 30 -c:v libx264 -pix_fmt yuv420p temp.mp4
	runwait %cmd%,,Hide

	; transcode with HandbrakeCLI
	cmd = %A_ScriptDir%\HandbrakeCli.exe -i temp.mp4 -o "%mp4OutputDir%\%output%.mp4"
	runwait %cmd%,,Hide

	; delete temporary files
	FileDelete,temp.mp4
	FileDelete,temp_png_*.png
}
FileDelete,temp.gif

exitapp

; display a window asking user whether to install or uninstall the script for each of
; the installed photobooth programs
Install()
{
	global DSLRRemoteState, NKRemoteState, WebcamState, PSRemoteState
	newCmd = "%A_AhkPath%" "%A_ScriptFullPath%"
	DSLRKey = Software\BreezeSystems\DSLRRemotePro\100
	NKRKey = Software\BreezeSystems\NKRemote\100
	WebKey = Software\BreezeSystems\WebcamPhotobooth\101
	PSRKey = Software\BreezeSystems\PSRemote\102
	Gui, Add, Text,, Install/uninstall GIF animation script
	Gui, Add, Text,, Please select item below to enable this script:
	RegRead, exe, HKEY_CURRENT_USER, %DSLRKey%, exe
	IfExist,%exe%
	{
		RegRead, enabled, HKEY_CURRENT_USER, %DSLRKey%, PhotoboothStatusCmdEnable
		RegRead, cmd, HKEY_CURRENT_USER, %DSLRKey%, PhotoboothStatusCmdXML

		if (cmd = newCmd and enabled = 1)
		{
			Gui, Add, Checkbox, gDSLRRemote checked vDSLRRemoteState, DSLR Remote Pro for Windows
		}
		else
		{
			Gui, Add, Checkbox, gDSLRRemote vDSLRRemoteState, DSLR Remote Pro for Windows
		}
	}
	RegRead, exe, HKEY_CURRENT_USER, %NKRKey%, exe
	IfExist,%exe%
	{
		RegRead, enabled, HKEY_CURRENT_USER, %NKRKey%, PhotoboothStatusCmdEnable
		RegRead, cmd, HKEY_CURRENT_USER, %NKRKey%, PhotoboothStatusCmdXML

		if (cmd = newCmd and enabled = 1)
		{
			Gui, Add, Checkbox, gNKRemote checked vNKRemoteState, NKRemote
		}
		else
		{
			Gui, Add, Checkbox, gNKRemote vNKRemoteState, NKRemote
		}
	}
	RegRead, exe, HKEY_CURRENT_USER, Software\BreezeSystems\WebcamPhotobooth\100, exe
	StringReplace,exe,exe,_,,1
	IfExist,%exe%
	{
		RegRead, enabled, HKEY_CURRENT_USER, %WebKey%, PhotoboothStatusCmdEnable
		RegRead, cmd, HKEY_CURRENT_USER, %WebKey%, PhotoboothStatusCmdXML

		if (cmd = newCmd and enabled = 1)
		{
			Gui, Add, Checkbox, gWebcam checked vWebcamState, Webcam Photobooth
		}
		else
		{
			Gui, Add, Checkbox, gWebcam vWebcamState, Webcam Photobooth
		}
	}
	RegRead, exe, HKEY_CURRENT_USER, Software\BreezeSystems\PSRemote\100, exe
	IfExist,%exe%
	{
		RegRead, enabled, HKEY_CURRENT_USER, %PSRKey%, PhotoboothStatusCmdEnable
		RegRead, cmd, HKEY_CURRENT_USER, %PSRKey%, PhotoboothStatusCmdXML

		if (cmd = newCmd and enabled = 1)
		{
			Gui, Add, Checkbox, gPSRemote checked vPSRemoteState, PSRemote
		}
		else
		{
			Gui, Add, Checkbox, gPSRemote vPSRemoteState, PSRemote
		}
	}

	Gui, Add, Button, Default, Close
	Gui, Show
	loop
	{
	}
DSLRRemote:
	Gui, Submit,NoHide
	if DSLRRemoteState
	{
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %DSLRKey%, PhotoboothStatusCmdEnable, 1
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %DSLRKey%, PhotoboothSuppressXml, 0
		RegWrite, REG_SZ, HKEY_CURRENT_USER, %DSLRKey%, PhotoboothStatusCmdXML, %newCmd%
		MsgBox,8192,%A_ScriptName%,Script enabled for DSLR Remote Pro,1000
	}
	else
	{
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %DSLRKey%, PhotoboothStatusCmdEnable, 0
		MsgBox,8192,%A_ScriptName%,Script disabled for DSLR Remote Pro,1000
	}
	return
NKRemote:
	Gui, Submit,NoHide
	if NKRemoteState
	{
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %NKRKey%, PhotoboothStatusCmdEnable, 1
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %NKRKey%, PhotoboothSuppressXml, 0
		RegWrite, REG_SZ, HKEY_CURRENT_USER, %NKRKey%, PhotoboothStatusCmdXML, %newCmd%
		MsgBox,8192,%A_ScriptName%,Script enabled for NKRemote,1000
	}
	else
	{
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %NKRKey%, PhotoboothStatusCmdEnable, 0
		MsgBox,8192,%A_ScriptName%,Script disabled for NKRemote,1000
	}
	return
Webcam:
	Gui, Submit,NoHide
	if WebcamState
	{
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %WebKey%, PhotoboothStatusCmdEnable, 1
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %WebKey%, PhotoboothSuppressXml, 0
		RegWrite, REG_SZ, HKEY_CURRENT_USER, %WebKey%, PhotoboothStatusCmdXML, %newCmd%
		MsgBox,8192,%A_ScriptName%,Script enabled for Webcam Photobooth,1000
	}
	else
	{
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %WebKey%, PhotoboothStatusCmdEnable, 0
		MsgBox,8192,%A_ScriptName%,Script disabled for Webcam Photobooth,1000
	}
	return
PSRemote:
	Gui, Submit,NoHide
	if PSRemoteState
	{
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %PSRKey%, PhotoboothStatusCmdEnable, 1
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %PSRKey%, PhotoboothSuppressXml, 0
		RegWrite, REG_SZ, HKEY_CURRENT_USER, %PSRKey%, PhotoboothStatusCmdXML, %newCmd%
		MsgBox,8192,%A_ScriptName%,Script enabled for PSRemote,1000
	}
	else
	{
		RegWrite, REG_DWORD, HKEY_CURRENT_USER, %PSRKey%, PhotoboothStatusCmdEnable, 0
		MsgBox,8192,%A_ScriptName%,Script disabled for PSRemote,1000
	}
	return

GuiClose:
ButtonClose:
	exitapp
}

DebugMessage(num, msg)
{
	global cStatusText, cStatusText2
	global showDebugMessages
	if showDebugMessages
	{
		if num=1
		{
			GuiControl,,StatusText,%msg%
		}
		else
		{
			GuiControl,,StatusText2,%msg%
		}
		Sleep 700
	}
}

